Ext.define('DSS.component.Editor', {
    extend: 'Ext.window.Window',

    /** Textfield holding the event name */
    editControls:{},
    buttonSave:null,
    buttonCancel:null,
    oTabPanelParent:null,
    innerHeight:427,
    makeControl: function (subConfig)
    {
    	var oControl;
    	switch (subConfig.type)
    	{
    		case 'hidden':
    		{
    			var oPseudoControl={value:null};
    			oPseudoControl.setValue=function(oValue){this.value=oValue;};
    			oPseudoControl.reset=function(oValue) {this.value=null;};
    			oPseudoControl.getValue=function() {return this.value;}
    			oPseudoControl.isValid=function() {return true;};
    			this.editControls[subConfig.id]=oPseudoControl;
    			return null;
    			break;
    		}
    		case 'groupbox':
			{
    			var oSubcontrols=[];
    			if (subConfig.content!=null)
	    			for (var iIndex=0;iIndex<subConfig.content.length;iIndex++)
	    				oSubcontrols.push(this.makeControl(subConfig.content[iIndex]));
    			
    			oControl = Ext.create('Ext.form.FieldSet', {    	          
    	            title: subConfig.label,
    	            items:oSubcontrols
    	        });
    			if ((subConfig.flex==null) || (!subConfig.flex)) 
            		this.innerHeight-=27;
        			
    			return oControl;
				break;
			}
    		case 'checkbox': 
			{
    			oControl = Ext.create('Ext.form.field.Checkbox', {
    	            name: subConfig.id,
    	            fieldLabel: subConfig.title,
    	            boxLabel:subConfig.label,
    	            allowBlank: false,
    	            width: 600,
    	            padding: "0 0 0 5"
    	        });
    			if ((subConfig.flex==null) || (!subConfig.flex))
    				this.innerHeight-=22;
    			this.editControls[subConfig.id]=oControl;
    	        return oControl;
				break;
			}
    		case 'text': 
			{    			
    			oControl = Ext.create('Ext.form.field.Text', {
    	            name: subConfig.id,
    	            fieldLabel: subConfig.label,
    	            allowBlank: false,
    	            width: 600
    	        });
    			if ((subConfig.flex==null) || (!subConfig.flex))
            		this.innerHeight-=27;
    			this.editControls[subConfig.id]=oControl;
    	        return oControl;
				break;
			}
       		case 'ActionConfigurator':
			{
    			oControl = Ext.create('DSS.component.ActionConfigurator',{height:350});
    			this.editControls[subConfig.id] = oControl;
    			if ((subConfig.flex==null) || (!subConfig.flex))
    				this.innerHeight-=350;
    			return oControl;
				break;
			}
    		case 'TriggerConfigurator':
			{
    			oControl = Ext.create('DSS.component.TriggerConfigurator',{height:350});
    			this.editControls[subConfig.id] = oControl;
    			if ((subConfig.flex==null) || (!subConfig.flex))
    				this.innerHeight-=350;
    			return oControl;
				break;
			}
    		case 'MessageControl':
			{
    			oControl = Ext.create('DSS.component.MessageEditor',{title:subConfig.title});
    			this.editControls[subConfig.id] = oControl;
    			if ((subConfig.flex==null) || (!subConfig.flex))
            		this.innerHeight-=150;

    			return oControl;
				break;
			}
    	}
    	return Ext.create('Ext.panel.Panel', {    		
    		border:'0'
    	});
    },  	

    constructor: function(config) {
    	config.closeAction= 'hide';         
    	config.modal= true; // Exclusive input for the window
    	config.height= 500;
    	config.width= 800;
    	config.layout= 'fit';
    	config.appName= '';
    	
        this.mixins.observable.constructor.call(config);
    
        var me = this;

        
        me.buttonCancel=Ext.create('Ext.Button', { 
        	text: _('Abort'),
            handler: function() {
                me.fireEvent('cancel');
                me.hide();
            }
        });
        
        me.buttonSave=Ext.create('Ext.Button', { 
        	text: _('Save'),
            handler: function() {
            	if (me.isValid()) {
                    me.fireEvent('save', me.getValues());
                    me.hide();
                }
            }
        });

        
        me.addEvents({
            cancel: true,
            save: true
        });

        var oStaticSubitems=[];
        var flexControl=null;
        if (config.staticControls!=null) {
        	for (var iIndex=0;iIndex<config.staticControls.length;iIndex++) {  
        		var oControl=me.makeControl(config.staticControls[iIndex]);
        		if (oControl!=null) {
        			oStaticSubitems.push(oControl);
        			oControl._myTabIndex=-1;
        			if (config.staticControls[iIndex].flex)
        				flexControl=oControl;
        		}
        	}        		
        }
        if (config.tabControls!=null) {
        	oTabPanelParent=Ext.create('Ext.tab.Panel', { 
        		height:this.innerHeight
        	});
        	this.innerHeight-=27;
        	var tabbedHeight=this.innerHeight;
        	for (var iIndex=0;iIndex<config.tabControls.length;iIndex++) {
        		flexControl=null;
        		this.innerHeight=tabbedHeight
        		if (config.tabControls[iIndex].content.length>0) {
	        		var oTabPanel=Ext.create('Ext.panel.Panel', {
	            		title:config.tabControls[iIndex].tabLabel,
	            		border:'0'
	            	});
	        		for (var jIndex=0;jIndex<config.tabControls[iIndex].content.length;jIndex++) {
	        			var oControl=me.makeControl(config.tabControls[iIndex].content[jIndex]);
	        			if (oControl!=null) {
	            			oControl._myTabIndex=iIndex;
	        				oTabPanel.add(oControl);
	        				if (config.tabControls[iIndex].content[jIndex].flex)
	            				flexControl=oControl;
	        			}
	        		}	        			
	        		oTabPanelParent.add(oTabPanel);
	            	if (flexControl!=null)
	            		flexControl.setHeight(this.innerHeight);
        		}
        	}   
        	oStaticSubitems.push(oTabPanelParent);
        } else {
        	if (flexControl!=null)
        		flexControl.setHeight(this.innerHeight);
        }
        		
        Ext.apply(me, {
            items: [{
            	xtype : 'panel',
            	items:oStaticSubitems,
            	bodyPadding: 5,
        		border:0,
                buttons: [ me.buttonCancel, me.buttonSave ]
            }], 
        }); // Ext.apply
        
        this.callParent(arguments);

    
    },

    setValues: function(values) {
        if (values ==null) {
        	for (var sKey in this.editControls)
        		if (this.editControls[sKey].reset!=null)
        			this.editControls[sKey].reset();
        		else
        			if (this.editControls[sKey].resetComponent!=null)
        				this.editControls[sKey].resetComponent();
        	
        } else {
            for (var sKey in this.editControls)
        	{
            	if (this.oTabPanelParent!=null)
            		if (this.editControls[sKey]._myTabIndex!=-1)
            			this.oTabPanelParent.setActiveTab(this.editControls[sKey]._myTabIndex);
        		if (values[sKey]!=null) {        			
        			this.editControls[sKey].setValue(values[sKey]);
        		}
        		else {
            		if (this.editControls[sKey].reset!=null)
            			this.editControls[sKey].reset();
            		else
            			if (this.editControls[sKey].resetComponent!=null)
            				this.editControls[sKey].resetComponent();
        		}
        	}
        }
    	if (this.oTabPanelParent!=null)
    		this.oTabPanelParent.setActiveTab(0);
    }, // resetComponent

    isValid: function() {
        var fisValid=true;
        for (var sKey in this.editControls)
    	{
    		if (!this.editControls[sKey].isValid())
    			fisValid= false;
    	}
        return fisValid;        
    },
   
    getValues: function() {
    	var oValues={};
    	for (var sKey in this.editControls)
    	{
    		oValues[sKey]=this.editControls[sKey].getValue();
    	}
        return oValues;
    }
});
